
abstract type SimulationModel end

struct MockModel <: SimulationModel
end

mintemp(model::MockModel) = 10.0
maxtemp(model::MockModel) = 15.0
function get_plant_growth_data(model)
    dds = [  # [5, n_inflections]
        [0.0, 10.0, 20.0],  # janfirst
        [0.0, 10.0, 20.0],  # sow
        [0.0, 10.0, 20.0],  # marchfirst
        [0.0, 10.0, 20.0],  # harvest1
        [0.0, 10.0, 20.0],  # harvest2
    ]
    slopes = [  # [5, 3, n_inflections]
        [  # janfirst
            [1.0, 2.0, 3.0],  # LAgreen
            [1.0, 2.0, 3.0],  # LAtotal
            [1.0, 2.0, 3.0],  # height
        ],
        [  # sow
            [1.0, 2.0, 3.0],  # LAgreen
            [1.0, 2.0, 3.0],  # LAtotal
            [1.0, 2.0, 3.0],  # height
        ],
        [  # marchfirst
            [1.0, 2.0, 3.0],  # LAgreen
            [1.0, 2.0, 3.0],  # LAtotal
            [1.0, 2.0, 3.0],  # height
        ],
        [  # harvest1
            [1.0, 2.0, 3.0],  # LAgreen
            [1.0, 2.0, 3.0],  # LAtotal
            [1.0, 2.0, 3.0],  # height
        ],
        [  # harvest2
            [1.0, 2.0, 3.0],  # LAgreen
            [1.0, 2.0, 3.0],  # LAtotal
            [1.0, 2.0, 3.0],  # height
        ],
    ]
    start = [  # [5, 3]
        [  # sow
            0.0,  # LAgreen
            0.0,  # LAtotal
            0.0,  # height
        ],
        [  # janfirst
            0.0,  # LAgreen
            0.0,  # LAtotal
            0.0,  # height
        ],
        [  # marchfirst
            0.0,  # LAgreen
            0.0,  # LAtotal
            0.0,  # height
        ],
        [  # harvest1
            0.0,  # LAgreen
            0.0,  # LAtotal
            0.0,  # height
        ],
        [  # harvest2
            0.0,  # LAgreen
            0.0,  # LAtotal
            0.0,  # height
        ],
    ]
    start_valid = [
        true,  # janfirst
        true,  # sow
        true,  # marchfirst
        true,  # harvest1
        true,  # harvest2
    ]
    cg = CropGrowth(; dds, slopes, start, start_valid)
    pgd = PlantGrowthData([cg], [1])
    return pgd
end

almass_SupplyGlobalRadiation(model::MockModel) = 1.0
almass_SupplyTemp(model::MockModel) = 12.0
almass_SolarConversion(model::MockModel, is_c4_plant, index) = 1.0
almass_SupplyFarmIntensity(model::MockModel, poly::Int) = 1.0

function diff(a::T, b::T) where {T}
    if ! isstructtype(T)
        error("Only works for struct types, type is $T")
    end
    println("Diff of $T structs:")
    for field in fieldnames(T)
        fa = getfield(a, field)
        fb = getfield(b, field)
        if fa != fb
            println("    - $field: ", fa)
            println("    + $field: ", fb)
        end
    end
end

const tov_Maize = 10
const tov_OMaizeSilage = 11
const tov_MaizeSilage = 12
const tov_MaizeStrigling = 13

include("new-almass.jl")

ve = VegElement()
model = MockModel()
ve_old = deepcopy(ve)

println(ve)
println()
almass_DoDevelopment(ve, model)
println(ve)
println()
diff(ve_old, ve)
